var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { Easing, StyleSheet, TextInput, Animated, View, } from 'react-native';
import { SuggesterContext } from './SuggesterContext';
import { DURATION, WINDOW_WIDTH, STATUS_BAR_HEIGHT, } from './Constants';
import { SuggesterEventEmitter } from './SuggesterEventEmitter';
import { setStateAsync } from './SetStateAsync';
export class SuggestTextInput extends Component {
    constructor(props) {
        super(props);
        this.state = {
            value: this.props.value,
            focused: false,
        };
        this.translateY = new Animated.Value(0);
        this.textInputRef = undefined;
        this.onSelect = (value) => __awaiter(this, void 0, void 0, function* () {
            yield setStateAsync({ component: this, state: { value } });
            const { onChangeText, onSubmitEditing } = this.props;
            if (onChangeText) {
                onChangeText(value);
            }
            if (onSubmitEditing) {
                onSubmitEditing({ nativeEvent: { text: value } });
            }
            this.textInputRef.current.blur();
        });
        this.handleFocus = ({ setMarginTopAsync, handleFocusProvider, setDataAsync, statusBarHeight, setPaddingHorizontalAsync, }) => (e) => __awaiter(this, void 0, void 0, function* () {
            const { onFocus } = this.props;
            if (onFocus) {
                onFocus(e);
            }
            const { name, data } = this.props;
            const { inputY, inputHeight, inputWidth } = yield this.measureAsync(this.textInputRef);
            yield setDataAsync(name, data);
            yield setMarginTopAsync(inputHeight);
            yield setPaddingHorizontalAsync((WINDOW_WIDTH - inputWidth) / 2);
            handleFocusProvider();
            this.setState({ focused: true });
            Animated.parallel([
                Animated.timing(this.translateY, {
                    toValue: -inputY + statusBarHeight || 0,
                    duration: DURATION,
                    easing: Easing.inOut(Easing.ease),
                    useNativeDriver: true,
                }),
            ]).start();
        });
        this.handleBlur = ({ handleBlurProvider }) => (e) => {
            const { onBlur } = this.props;
            if (onBlur) {
                onBlur(e);
            }
            handleBlurProvider();
            Animated.parallel([
                Animated.timing(this.translateY, {
                    toValue: 0,
                    duration: DURATION,
                    easing: Easing.inOut(Easing.ease),
                    useNativeDriver: true,
                }),
            ]).start(({ finished }) => {
                if (finished) {
                    this.setState({ focused: false });
                }
            });
        };
        this.handleChange = ({ setValueAsync }) => (value) => {
            const { name, onChangeText } = this.props;
            if (onChangeText) {
                onChangeText(value);
            }
            setValueAsync(name, value);
            this.setState({ value });
        };
        this.measureAsync = (ref) => new Promise(resolve => {
            if (ref && ref.current) {
                ref.current.measure((_fx, _fy, width, height, _px, py) => {
                    resolve({
                        inputY: py,
                        inputHeight: height,
                        inputWidth: width,
                    });
                });
            }
        });
        this.handleSubmit = (e) => {
            const { onSubmitEditing, onChangeText } = this.props;
            if (onSubmitEditing) {
                onSubmitEditing(e);
            }
            if (onChangeText) {
                onChangeText(this.state.value);
            }
        };
        this.textInputRef = React.createRef();
    }
    componentDidMount() {
        const { name } = this.props;
        SuggesterEventEmitter.on(`selectFromList-${name}`, this.onSelect);
    }
    componentWillUnmount() {
        const { name } = this.props;
        SuggesterEventEmitter.off(`selectFromList-${name}`, this.onSelect);
    }
    render() {
        const { translateY } = this;
        const { value, focused } = this.state;
        const opacity = translateY.interpolate({
            inputRange: [0, 1],
            outputRange: [1, 0],
        });
        return (<SuggesterContext.Consumer>
        {({ setMarginTopAsync, handleFocusProvider, handleBlurProvider, setDataAsync, statusBarHeight, backgroundColor, setValueAsync, setPaddingHorizontalAsync, }) => (<Animated.View style={[
            {
                backgroundColor,
                transform: [{ translateY }],
                zIndex: 2000,
            },
            focused
                ? {
                    backgroundColor,
                    opacity,
                    width: WINDOW_WIDTH,
                    justifyContent: 'center',
                    alignItems: 'center',
                }
                : { backgroundColor: 'transparent' },
        ]}>
            {focused && (<View style={Object.assign({}, StyleSheet.absoluteFillObject, { marginTop: -STATUS_BAR_HEIGHT, backgroundColor, height: STATUS_BAR_HEIGHT, width: WINDOW_WIDTH })}/>)}
            <TextInput autoCorrect={false} {...this.props} autoFocus={false} ref={this.textInputRef} value={value} onChangeText={this.handleChange({ setValueAsync })} onFocus={this.handleFocus({
            setMarginTopAsync,
            handleFocusProvider,
            setDataAsync,
            statusBarHeight,
            setPaddingHorizontalAsync,
        })} onSubmitEditing={this.handleSubmit} onBlur={this.handleBlur({ handleBlurProvider })}/>
          </Animated.View>)}
      </SuggesterContext.Consumer>);
    }
}
SuggestTextInput.propTypes = {
    name: PropTypes.string.isRequired,
    data: PropTypes.arrayOf(PropTypes.shape({
        id: PropTypes.oneOfType([PropTypes.number, PropTypes.string])
            .isRequired,
        value: PropTypes.string.isRequired,
    }).isRequired),
};
//# sourceMappingURL=SuggestTextInput.js.map